/**
 * InventoryFermentables.cpp is part of bmsapp.
 *
 * bmsapp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * bmsapp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "InventoryFermentables.h"
#include "EditFermentable.h"
#include "../ui/ui_InventoryFermentables.h"
#include "config.h"
#include "bmsapp.h"


InventoryFermentables::InventoryFermentables(QWidget *parent) : QDialog(parent), ui(new Ui::InventoryFermentables)
{
    qDebug() << "InventoryFermentables start";

    ui->setupUi(this);
    emit refreshTable();

    setWindowTitle( QString("BMSapp - %1 - Inventory Fermentables").arg(VERSIONSTRING) );
}


void InventoryFermentables::refreshTable()
{
    QString w;

    qDebug() << "InventoryFermentables reload";

    QSqlQuery query("SELECT * FROM inventory_fermentables ORDER BY supplier,name");
    const QStringList labels({tr("Origin"), tr("Supplier"), tr("Name"), tr("Type"), tr("Grain"), tr("Color"), tr("Yield"), tr("Stock"), tr("Edit")});
    const QStringList types({tr("Grain"), tr("Sugar"), tr("Extract"), tr("Dry extract"), tr("Adjunct")});
    const QStringList graintypes({tr("Base"), tr("Roast"), tr("Crystal"), tr("Kilned"), tr("Sour Malt"), tr("Special"), tr("No malt")});

    /* origin supplier name type graintype color yield inventory Edit */
    ui->tableFermentables->setColumnCount(9);
    ui->tableFermentables->setColumnWidth(0, 130);	/* Origin	*/
    ui->tableFermentables->setColumnWidth(1, 170);	/* Supplier	*/
    ui->tableFermentables->setColumnWidth(2, 250);	/* Name		*/
    ui->tableFermentables->setColumnWidth(3,  90);	/* Type		*/
    ui->tableFermentables->setColumnWidth(4,  90);	/* Graintype	*/
    ui->tableFermentables->setColumnWidth(5,  80);	/* Color	*/
    ui->tableFermentables->setColumnWidth(6,  80);	/* Yield	*/
    ui->tableFermentables->setColumnWidth(7,  80);	/* Stock	*/
    ui->tableFermentables->setColumnWidth(8,  80);	/* Edit button	*/
    ui->tableFermentables->setRowCount(query.size());
    ui->tableFermentables->setHorizontalHeaderLabels(labels);
    ui->tableFermentables->verticalHeader()->hide();
    ui->tableFermentables->setFixedSize(1050 + 24, 640);	/* Even if this is too large, it works */

    QTableWidgetItem *rightitem = new QTableWidgetItem();
    rightitem->setTextAlignment(Qt::AlignRight);

    query.first();
    for (int ridx = 0 ; ridx < query.size() ; ridx++ ) {
	ui->tableFermentables->setItem(ridx, 0, new QTableWidgetItem(query.value(6).toString()));
	ui->tableFermentables->setItem(ridx, 1, new QTableWidgetItem(query.value(7).toString()));
	ui->tableFermentables->setItem(ridx, 2, new QTableWidgetItem(query.value(1).toString()));
	ui->tableFermentables->setItem(ridx, 3, new QTableWidgetItem(types[query.value(2).toInt()]));
	ui->tableFermentables->setItem(ridx, 4, new QTableWidgetItem(graintypes[query.value(20).toInt()]));
	w = QString("%1 EBC").arg(query.value(4).toDouble(), 1, 'f', 0, '0' );
	QTableWidgetItem *item = new QTableWidgetItem(w);
	item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
        ui->tableFermentables->setItem(ridx, 5, item);
	w = QString("%1 %").arg(query.value(3).toFloat(), 3, 'f', 1, '0' );
	item = new QTableWidgetItem(w);
	item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
	ui->tableFermentables->setItem(ridx, 6, item);
	w = QString("");
	if (query.value(21).toDouble() > 0) {
	  if (query.value(21).toDouble() < 1.000) {
	    w = QString("%1 gr").arg(query.value(21).toDouble() * 1000.0, 1, 'f', 0, '0' );
	  } else {
            w = QString("%1 kg").arg(query.value(21).toDouble(), 4, 'f', 3, '0' );
	  }
	}
	item = new QTableWidgetItem(w);
	item->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
	ui->tableFermentables->setItem(ridx, 7, item);

	/* Add the Edit button */
	QWidget* pWidget = new QWidget();
	QPushButton* btn_edit = new QPushButton();
	btn_edit->setObjectName(QString("%1").arg(query.value(0).toString()));	/* Send record with the button */
	btn_edit->setText(tr("Edit"));
	connect(btn_edit, SIGNAL(clicked()), this, SLOT(on_editButton_clicked()));
	QHBoxLayout* pLayout = new QHBoxLayout(pWidget);
	pLayout->addWidget(btn_edit);
	pLayout->setContentsMargins(5, 0, 5, 0);
	pWidget->setLayout(pLayout);
	ui->tableFermentables->setCellWidget(ridx, 8, pWidget);
	query.next();
    }

    setWindowTitle( QString("BMSapp - %1 - Inventory Fermentables").arg(VERSIONSTRING) );
}


InventoryFermentables::~InventoryFermentables()
{
    qDebug() << "InventoryFermentables done";
    delete ui;
}


void InventoryFermentables::edit(int recno)
{
    qDebug() << "InventoryFermentables edit:" << recno;

    EditFermentable dialog(recno, this);
    /* Signal from editor if a refresh is needed */
    connect(&dialog, SIGNAL(entry_changed()), this, SLOT(refreshTable()));
    dialog.setModal(true);
    dialog.exec();
}


void InventoryFermentables::on_editButton_clicked()
{
    QPushButton *pb = qobject_cast<QPushButton *>(QObject::sender());
    int recno = pb->objectName().toInt();
    qDebug() << Q_FUNC_INFO << recno;
    edit(recno);
}


void InventoryFermentables::on_insertButton_clicked()
{
    qDebug() << Q_FUNC_INFO;
    edit(-1);
}


void InventoryFermentables::on_exportButton_clicked()
{
    qDebug() << Q_FUNC_INFO;

    QSqlQuery query("SELECT * FROM inventory_fermentables ORDER BY supplier,name");
    const QStringList types({"Grain", "Sugar", "Extract", "Dry extract", "Adjunct"});
    const QStringList graintypes({"Base", "Roast", "Crystal", "Kilned", "Sour Malt", "Special", "No malt"});
    const QStringList added({"Mash", "Boil", "Fermentation", "Lagering", "Bottle"});

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"), QDir::homePath() + "/fermentables.xml", tr("Files (*.xml)"));
    if (fileName == 0) {
	QMessageBox::warning(this, tr("Save File"), tr("No XML file selected."));
        return;
    }

    QFile file(fileName);
    file.open(QIODevice::WriteOnly);

    QXmlStreamWriter *xmlWriter = new QXmlStreamWriter(&file);
    xmlWriter->writeStartDocument();
    xmlWriter->setAutoFormatting(true);
    xmlWriter->setAutoFormattingIndent(1);
    xmlWriter->writeStartElement("FERMENTABLES");

    query.first();
    for (int i = 0 ; i < query.size() ; i++ ) {
	xmlWriter->writeStartElement("FERMENTABLE");
	xmlWriter->writeTextElement("VERSION", "1");
	xmlWriter->writeTextElement("NAME", query.value(1).toString());
	xmlWriter->writeTextElement("NOTES", query.value(8).toString());
	xmlWriter->writeTextElement("TYPE", types[query.value(2).toInt()]);
	xmlWriter->writeTextElement("YIELD", QString::number(query.value(3).toDouble(), 'f', 5));
	xmlWriter->writeTextElement("COLOR", QString::number(Utils::ebc_to_srm(query.value(4).toDouble()), 'f', 3) );
	xmlWriter->writeTextElement("ADD_AFTER_BOIL", query.value(5).toInt() ? "TRUE":"FALSE");
	xmlWriter->writeTextElement("ORIGIN", query.value(6).toString());
	xmlWriter->writeTextElement("SUPPLIER", query.value(7).toString());
	if (query.value(9).toDouble() > 0)
	    xmlWriter->writeTextElement("COARSE_FINE_DIFF", QString::number(query.value(9).toDouble(), 'f', 4));
	if (query.value(10).toDouble() > 0)
	    xmlWriter->writeTextElement("MOISTURE", QString::number(query.value(10).toDouble(), 'f', 4));
	if (query.value(11).toDouble() > 0)
	    xmlWriter->writeTextElement("DIASTATIC_POWER", QString::number(query.value(11).toDouble(), 'f', 4));
	if (query.value(12).toDouble() > 0)
	    xmlWriter->writeTextElement("PROTEIN", QString::number(query.value(12).toDouble(), 'f', 4));
	if (query.value(14).toDouble() > 0)
	    xmlWriter->writeTextElement("MAX_IN_BATCH", QString::number(query.value(14).toDouble(), 'f', 3));
	xmlWriter->writeTextElement("RECOMMEND_MASH", query.value(15).toInt() ? "TRUE":"FALSE");
	xmlWriter->writeTextElement("DISPLAY_COLOR", QString("%1 EBC").arg(query.value(4).toDouble(), 1, 'f', 0));
	xmlWriter->writeTextElement("GRAINTYPE", graintypes[query.value(20).toInt()]);
	xmlWriter->writeTextElement("ADDED", added[query.value(17).toInt()]);
	if (query.value(18).toDouble() > 0)
	    xmlWriter->writeTextElement("DI_pH", QString::number(query.value(18).toDouble(), 'f', 5));
	if (query.value(19).toDouble() > 0)
	    xmlWriter->writeTextElement("ACID_TO_pH_5.7", QString::number(query.value(19).toDouble(), 'f', 5));

	xmlWriter->writeEndElement();
	query.next();
    }

    xmlWriter->writeEndElement();
    xmlWriter->writeEndDocument();
    QMessageBox::information(this, tr("Save File"), tr("XML export ready"));

    file.close();
}


void InventoryFermentables::on_quitButton_clicked()
{
    emit firstWindow();
}

